﻿
// Dundas BI Sample Code
// 2018 Dundas Data Visualization Inc.


using Dundas.BI.Data.DataProviders;
using Dundas.BI.Data.Parameters;
using Dundas.BI.Export;
using Dundas.BI.Notifications;
using Dundas.BI.Notifications.Delivery;

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO.Compression;
using System.Linq;
using System.Net.Mime;

namespace SlackSample
{
    /// <summary>
    /// This class extends delivery provider in order to create a slack devivery provider.
    /// </summary>
    /// <seealso cref="Dundas.BI.Notifications.Delivery.DeliveryProvider" />
    public class SlackDeliveryProvider : DeliveryProvider
	{
        #region Public Properties

        /// <summary>
        /// Gets the component description.
        /// </summary>
        public override string ComponentDescription
		{
			get
			{
				return "Slack Delivery Provider Sample that provides delivery to slack channels";
			}
		}

        /// <summary>
        /// Gets the component ID.
        /// </summary>
        public override Guid ComponentId
		{
			get
			{
				return new Guid("03771c09-44eb-4810-9a85-671a18ead53d");
			}
		}

        /// <summary>
        /// Gets the component name.
        /// </summary>
        public override string ComponentName
		{
			get
			{

				return "Slack";
			}
		}

        /// <summary>
        /// Gets the kind of delivery the provider provides.
        /// </summary>
        public override NotificationDeliveryKind DeliveryKind
        {
	        get
	        {
		        return NotificationDeliveryKind.ExtensionProvider;
	        }
        }

        /// <summary>
        /// Gets a value indicating whether this delivery provider is using a custom configuration UI.
        /// </summary>
        public override bool HasCustomConfigurationUI
		{
			get
			{
				return true;
			}
		}

        /// <summary>
        /// Gets the delivery provider specific property descriptors.
        /// </summary>
        public override IReadOnlyList<ProviderProperty> PropertyDescriptors
		{
			get
			{
				return SlackHelper.SlackProperties;
			}
		}

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Delivers a set of export results.
        /// </summary>
        /// <param name="request">The delivery request.</param>
        /// <param name="exportResults">The set of export results to deliver.</param>
        public override void Deliver(DeliveryRequest request, ICollection<ExportResult> exportResults)
        {
            var channelValue =
                request.DeliverySettings.ProviderSettings.FirstOrDefault(
                    deliverySettingItem =>
                        deliverySettingItem.ParameterId.Equals(
                            SlackExtensionProviderSettingIds.SlackChannelPropertyId
                            )
                    );

            var titleValue =
                request.DeliverySettings.ProviderSettings.FirstOrDefault(
                    deliverySettingItem =>
                        deliverySettingItem.ParameterId.Equals(
                            SlackExtensionProviderSettingIds.SlackTitlePropertyId
                            )
                    );

            foreach (ExportResult exportResult in exportResults)
	        {

                string fileExtension = exportResult.FileExtension;
                byte[] bytes = exportResult.Content.ToByteArray();
               
                SlackHelper.PostFile(
                    bytes,
                    string.Format(CultureInfo.InvariantCulture, "{0}.{1}", exportResult.Name, exportResult.FileExtension),
                    fileExtension,
                    channelValue.ToString(),
                    titleValue.ToString(),
                    request.ActionText
                    );
                
	        }
        }

        /// <summary>
        /// Gets the custom configuration UI.
        /// </summary>
        /// <param name="contentType">The content type that the UI is requested for.</param>
        /// <returns>
        /// The string holding the custom UI, or <see langword="null" /> if none is supported for the content type.
        /// </returns>
        public override string GetCustomConfigurationUI(ContentType contentType)
		{

            if (contentType == null)
            {
                return null;
            }

            // We support html.
            if (contentType.Equals(MediaTypeNames.Text.Html))
            {
                return SlackHelper.SlackDeliveryUI;
            }

            return null;

            
		}

		#endregion Public Methods
	}
}