﻿
// Dundas BI Sample Code
// 2018 Dundas Data Visualization Inc.


using Dundas.BI;
using Dundas.BI.Data.DataProviders;
using Dundas.BI.Export;
using Dundas.BI.Services;

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Mime;
using Dundas.BI.Data.Parameters;

namespace SlackSample
{
    /// <summary>
    /// This class extends export provider in order to create a slack export provider.
    /// </summary>
    /// <seealso cref="Dundas.BI.Export.ExportProvider" />
    public class SlackExportProvider : ExportProvider
	{
        #region Public Properties

        /// <summary>
        /// Gets a value indicating whether this export provider creates a file or not.
        /// </summary>
        public override bool CreatesFile
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the standard component description.
        /// </summary>
        public override string ComponentDescription
		{
			get
			{
				return "Slack export sample that provides export from Dundas BI to a Slack channel.";
			}
		}

        /// <summary>
        /// Gets the component ID.
        /// </summary>
        public override Guid ComponentId
		{
			get
			{
				return new Guid("37f781f6-1fa7-4b82-a410-3031de28ac42");
			}
		}

        /// <summary>
        /// Gets the component name.
        /// </summary>
        public override string ComponentName
		{
			get
			{
				return "Slack";
			}
		}

        /// <summary>
        /// Gets the file extension of the export result.
        /// </summary>
        public override string FileExtension
		{
			get
			{
				return string.Empty;
			}
		}

		public override Uri IconUri
		{
			get
			{
                return new Uri(
                    "../Content/Override/Images/Slack/slack.png",
                    UriKind.Relative
                    );

			}
		}

        /// <summary>
        /// Gets the MIME type of the export result.
        /// </summary>
        public override string MimeType
		{
			get
			{
				return string.Empty;
			}
		}

        /// <summary>
        /// Gets a value indicating whether this export provider is using a custom configuration UI.
        /// </summary>
        public override bool HasCustomConfigurationUI
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Gets the custom configuration UI based on the requested content type.
        /// </summary>
        /// <param name="contentType">The content type that the UI is requested for.</param>
        /// <returns>
        /// The string holding the custom UI, or <see langword="null" /> if none is supported for the content type.
        /// </returns>
        public override string GetCustomConfigurationUI(ContentType contentType)
        {
            if (contentType == null)
            {
                return null;
            }

            // We support html.
            if (contentType.Equals(MediaTypeNames.Text.Html))
            {
                return SlackHelper.SlackExportUI;
            }

            return null;
        }

        /// <summary>
        /// Gets the export provider specific property descriptors.
        /// </summary>
        public override IReadOnlyList<ProviderProperty> PropertyDescriptors
		{
			get
			{
                return SlackHelper.SlackProperties;
            }
		}

        /// <summary>
        /// Gets a value indicating whether this export provider is available from share, notifications or both.
        /// </summary>
        public override ExportProviderKinds ProviderKinds
		{
			get
			{
				return ExportProviderKinds.Share;
			}
		}

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Performs an export.
        /// </summary>
        /// <param name="request">The export request.</param>
        /// <returns>
        /// The export result.
        /// </returns>
        public override ExportResult Export(ExportRequest request)
        {

	        Dundas.BI.Export.StandardExportProviders.PngProvider pngProvider = 
                new Dundas.BI.Export.StandardExportProviders.PngProvider();

	        ExportRequest exportRequest =
		        new ExportRequest(
                    Dundas.BI.Export.StandardExportProviders.StandardExportProviderIds.Png, 
                    request.ViewId, 
                    request.ViewParameters
                    );

	        ExportResult result =
		        pngProvider.Export(exportRequest);

            var channelValue =
                request.ProviderSettings.FirstOrDefault(
                    deliverySettingItem =>
                        deliverySettingItem.ParameterId.Equals(
                            SlackExtensionProviderSettingIds.SlackChannelPropertyId
                            )
                    );

            var titleValue =
                request.ProviderSettings.FirstOrDefault(
                    deliverySettingItem =>
                        deliverySettingItem.ParameterId.Equals(
                            SlackExtensionProviderSettingIds.SlackTitlePropertyId
                            )
                    );

            var initialCommentValue =
                request.ProviderSettings.FirstOrDefault(
                    deliverySettingItem =>
                        deliverySettingItem.ParameterId.Equals(
                            SlackExtensionProviderSettingIds.SlackInitialCommentPropertyId
                            )
                    );



            SlackHelper.PostFile( 
                result.Content.ToByteArray(), 
                string.Format(
                    CultureInfo.InvariantCulture, 
                    "{0}.png", 
                    result.Name
                    ),
                "png", 
                channelValue.ToString(),
                titleValue.ToString(),
                initialCommentValue.ToString()
                );

	        return result;
        }

		#endregion Public Methods
	}
}