﻿
// Dundas BI Sample Code
// 2018 Dundas Data Visualization Inc.


using Dundas.BI;
using Dundas.BI.Configuration;
using Dundas.BI.Data.DataProviders;
using Dundas.BI.Data.Parameters;
using SlackAPI;

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace SlackSample
{
    /// <summary>
    /// Contains helper functions for connecting to and using Slack.
    /// </summary>
    public static class SlackHelper
	{
		#region Static Fields

		public static TimeSpan SlackTimeout = new TimeSpan(0, 1, 0);

        #endregion Static Fields

        #region Internal Properties

        /// <summary>
        /// Gets the HTML used to define the Slack export UI.
        /// </summary>
        /// <value>
        /// The HTML used to define the Slack export UI.
        /// </value>
        internal static string SlackExportUI
        {
            get
            {
                return string.Concat(
                    SlackHelper.ChannelsUI,
                    SlackHelper.SlackTitleUI,
                    SlackHelper.SlackInitialCommentUI
                    );
            }
        }

        /// <summary>
        /// Gets the HTML used to define the Slack delivery UI.
        /// </summary>
        /// <value>
        /// The HTML used to define the Slack delivery UI.
        /// </value>
        internal static string SlackDeliveryUI
        {
            get
            {
                return string.Concat(
                    SlackHelper.ChannelsUI,
                    SlackHelper.SlackTitleUI
                    );
            }
        }

        /// <summary>
        /// Gets the HTML used to define the slack title propery.
        /// </summary>
        /// <value>
        /// The HTML used to define the slack title propery.
        /// </value>
        internal static string SlackTitleUI
        {
            get
            {
                return
                    GenerateSimpleStringUI(
                        SlackExtensionProviderSettingIds.SlackTitlePropertyId,
                        "Title (Optional)"
                        );
            }
        }

        /// <summary>
        /// Gets the slack initial comment UI.
        /// </summary>
        /// <value>
        /// The slack initial comment UI.
        /// </value>
        internal static string SlackInitialCommentUI
        {
            get
            {
                return
                    GenerateSimpleStringUI(
                        SlackExtensionProviderSettingIds.SlackInitialCommentPropertyId,
                        "Initial Comment (Optional)", 
                        true
                        );
            }
        }

        /// <summary>
        /// Gets the HTML used to define the channels property.
        /// </summary>
        /// <value>
        /// The HTML used to define the channels property.
        /// </value>
        internal static string ChannelsUI
        {
            get
            {
                StringBuilder uiBuilder = new StringBuilder();

                uiBuilder.Append("<div>");

                uiBuilder.Append("\t<p>Slack Channel</p>");
                
                uiBuilder.AppendLine();
                uiBuilder.Append("\t<select title=\"Set the slack channel to post to:\" id=\"");
                uiBuilder.Append(SlackExtensionProviderSettingIds.SlackChannelPropertyId);
                uiBuilder.Append("\" name=\"");
                uiBuilder.Append(SlackExtensionProviderSettingIds.SlackChannelPropertyId);
                uiBuilder.Append("\" data-valuetype=\"SingleString\" >");

                foreach (Channel channel in

                    SlackHelper.GetSlackChannels())
                {
                    uiBuilder.AppendLine();
                    uiBuilder.Append("\t\t<option value=\"");
                    uiBuilder.Append(channel.name);
                    uiBuilder.Append("\" >");
                    uiBuilder.Append(channel.name);
                    uiBuilder.Append("</option>");
                    uiBuilder.AppendLine();
                }

                uiBuilder.AppendLine("</select>");

                uiBuilder.Append("</div>");

                return uiBuilder.ToString();

            }
        }

        /// <summary>
        /// Gets the slack password.
        /// </summary>
        /// <value>
        /// The slack password.
        /// </value>
        internal static string SlackPassword
		{
			get
			{
				return
					Engine.Current.GetService<IAppConfigService>().GetString(
						SlackExtensionProviderSettingIds.SlackPasswordId
						);
			}
		}

        /// <summary>
        /// Gets the slack team identifier.
        /// </summary>
        /// <value>
        /// The slack team identifier.
        /// </value>
        internal static string SlackTeamId
		{
			get
			{
				return
					Engine.Current.GetService<IAppConfigService>().GetString(
						SlackExtensionProviderSettingIds.SlackTeamIdId
						);
			}
		}

        /// <summary>
        /// Gets the slack user identifier.
        /// </summary>
        /// <value>
        /// The slack user identifier.
        /// </value>
        internal static string SlackUserId
		{
			get
			{
				return
					Engine.Current.GetService<IAppConfigService>().GetString(
						SlackExtensionProviderSettingIds.SlackUserIdId
						);
			}
		}

/// <summary>
/// Gets the slack properties.
/// </summary>
/// <value>
/// The slack properties.
/// </value>
public static IReadOnlyList<ProviderProperty> SlackProperties
{
    get
    {
        List<ProviderProperty> properties = new List<ProviderProperty>();

        ProviderProperty propertyDescriptor =
            new StringProperty(SlackExtensionProviderSettingIds.SlackChannelPropertyId,
            "Slack Channel",
            "The slack channel to post to",
            "general", ValidValuesSource.None);
        propertyDescriptor.IsValueRequired = true;

        properties.Add(propertyDescriptor);


        ProviderProperty titlePropertyDescriptor =
            new StringProperty(SlackExtensionProviderSettingIds.SlackTitlePropertyId,
            "Slack Title",
            "The slack title of the Slack post",
            string.Empty, ValidValuesSource.None);
        propertyDescriptor.IsValueRequired = false;

        properties.Add(titlePropertyDescriptor);

        ProviderProperty initialCommentPropertyDescriptor =
            new StringProperty(SlackExtensionProviderSettingIds.SlackTitlePropertyId,
            "Slack Initial Comment",
            "The commeent of the Slack post",
            string.Empty, ValidValuesSource.None);
        propertyDescriptor.IsValueRequired = false;

        properties.Add(initialCommentPropertyDescriptor);

        return properties;
    }
}

        #endregion Internal Properties

        #region Public Methods


        private static string GetAccessToken()
		{
			return GetAccessToken(SlackUserId, SlackTeamId, SlackPassword);
		}


        private static string GetAccessToken(string userid, string teamid, string password)
		{
			DateTime startDateTime = DateTime.Now;

			AuthSigninResponse authSigninResponse = null;

			Action<AuthSigninResponse> accessTokenReponseCallback =
				new Action<AuthSigninResponse>(tokenResponse =>
				{
					authSigninResponse = tokenResponse;
				});

			SlackClient.AuthSignin(accessTokenReponseCallback, userid, teamid, password);

			while (authSigninResponse == null)
			{
				if(DateTime.Now - startDateTime > SlackTimeout)
				{
					throw new TimeoutException();
				}

				Thread.Sleep(1000);
			}

			if (authSigninResponse.ok)
			{
				return authSigninResponse.token;
			}
			else
			{
				throw new SlackException(authSigninResponse.error);
			}
		}

        /// <summary>
        /// Gets the slack channels.
        /// </summary>
        /// <returns>An array of slack channels.</returns>
        /// <exception cref="TimeoutException"></exception>
        public static Channel[] GetSlackChannels()
		{
			DateTime startDateTime = DateTime.Now;

            string accessToken = SlackHelper.GetAccessToken();

			SlackClient client = new SlackClient(accessToken);

			ChannelListResponse channelListResponse = null;

			Action<ChannelListResponse> channelListResponseAction =
				new Action<ChannelListResponse>(channelListResponseItem => { channelListResponse = channelListResponseItem; });

			client.GetChannelList(channelListResponseAction);

			while (channelListResponse == null)
			{
				if (DateTime.Now - startDateTime > SlackTimeout)
				{
					throw new TimeoutException();
				}

				Thread.Sleep(1000);
			}

			channelListResponse.AssertOk();

			return channelListResponse.channels;
		}

        /// <summary>
        /// Posts a file to Slack.
        /// </summary>
        /// <param name="fileBytes">The file bytes.</param>
        /// <param name="fileName">Name of the file.</param>
        /// <param name="fileExtension">The file extension.</param>
        /// <param name="channelName">Name of the slack channel.</param>
        /// <param name="title">The title.</param>
        /// <param name="initialComment">The initial comment.</param>
        /// <exception cref="TimeoutException">
        /// </exception>
        /// <exception cref="SlackException"></exception>
        public static void PostFile(byte[] fileBytes, string fileName, string fileExtension, string channelName, string title = "", string initialComment = "")
		{
			DateTime startDateTime = DateTime.Now;

            string accessToken = SlackHelper.GetAccessToken();

            SlackClient client = new SlackClient(accessToken);

			FileUploadResponse fileUploadResponse = null;

			Action<FileUploadResponse> fileUploadResponseCallback =
				new Action<FileUploadResponse>(uploadResponse =>
				{
					fileUploadResponse = uploadResponse;
				});


			ChannelListResponse clr = null;

			client.GetChannelList(new Action<ChannelListResponse>(clrItem => { clr = clrItem; }));

			while (clr == null)
			{
				if (DateTime.Now - startDateTime > SlackTimeout)
				{
					throw new TimeoutException();
				}

				Thread.Sleep(1000);
			}



			string channelId = clr.channels.FirstOrDefault(ci => ci.name.Equals(channelName)).id;

			client.UploadFile(
				fileUploadResponseCallback,
				fileBytes,
				fileName,
				new string[] { channelId },
				title,
				initialComment,
				false,
				fileExtension
				);

			while (fileUploadResponse == null)
			{
				if (DateTime.Now - startDateTime > SlackTimeout)
				{
					throw new TimeoutException();
				}

				Thread.Sleep(1000);
			}

			if (!fileUploadResponse.ok)
			{
				throw new SlackException(fileUploadResponse.error);
			}
		}



        /// <summary>
        /// Generates the simple string UI.
        /// </summary>
        /// <param name="propertyId">The property identifier.</param>
        /// <param name="nameOfProperty">The name of property.</param>
        /// <returns>An HTML string for the property specified.</returns>
        public static string GenerateSimpleStringUI(
            Guid propertyId, string nameOfProperty, bool isMultiline = false
            )
        {


            if (isMultiline)
            {
                return string.Format(
                    CultureInfo.CurrentCulture,
                    @"<div>
                        <p>{1}</p>
                        <textarea 
                           rows=""5"" 
                           name=""{0}"" 
                           id=""{0}"" 
                           data-valuetype=""SingleString"" />
                        </div>",
                    propertyId,
                    nameOfProperty
                    );
            }
            else
            {
                return string.Format(
                    CultureInfo.CurrentCulture,
                    @"<div>
                        <p>{1}</p>
                        <input 
                            type=""text"" 
                            name=""{0}"" 
                            id=""{0}"" 
                            data-valuetype=""SingleString"" />
                        </div>",
                    propertyId,
                    nameOfProperty
                    );
            }
        }

        #endregion Public Methods
    }
}